const snippetTypeRegex = /\[(code|render)\s([\w_\/-]+)(\s.*)?\]/gi;
const snippetGroupTypeRegex = /\[(code|render)-group\](?:\s*)(\[[\s\S]*?\])(?:\s*)\[\/\1-group\]/gi;
const snippetRegex = /\[([\w_\/-]+)\s*([\s\S]*?)\]/gi;
const paramRegex = /([\w_-]+="[^"]+")/gi;

var shortcode = function () {

  function getParams(params) {
    var paramsObj = {};
    if (params) {
      params.match(paramRegex).forEach((param) => {
        var _paramsArr = param.split('=');
        paramsObj[_paramsArr[0]] = _paramsArr[1].replace(/"(.+)"/g, '$1'); // remove double quotes
      });
    }
    return paramsObj;
  }

  return [{
    type: 'lang',
    filter: function (text, converter, options) {
      return text.replace(snippetTypeRegex, function (match, type, template, params) {
        return `<%- ${type}('${template}', ${JSON.stringify(getParams(params))}) %>`;
      });
    }
  }, {
    type: 'lang',
    filter: function (text, converter, options) {
      return text.replace(snippetGroupTypeRegex, function (match, type, snippets) {
        var snippetsArr = [];
        snippets.replace(snippetRegex, function (match, template, params) {
          snippetsArr.push({
            template: template,
            data: getParams(params)
          });
        });
        return `<%- ${type}(${JSON.stringify(snippetsArr)}) %>`;
      });
    }
  }];

};

module.exports = shortcode;
